import subprocess

import docker
import typer
from yaspin import yaspin

from rubbrband.controllers import eval, train

__author__ = "Rubbrband"

app = typer.Typer(no_args_is_help=True, rich_markup_mode="rich")
app.add_typer(train.app, name="train", subcommand_metavar="MODEL")
app.add_typer(eval.app, name="eval", subcommand_metavar="MODEL")

try:
    client = docker.from_env()
except docker.errors.DockerException:
    typer.echo("Docker is not running. Please start Docker and try again.")
    exit()


# create our database of models
db = {
    "lora": {
        "description": "Low-rank adapation for efficient stable diffusion fine tuning",
        "shape": "anything",
    },
    "dreambooth": {
        "description": "Stable diffusion models, trained with dreambooth method",
        "shape": "anything",
    },
    "control": {
        "description": "Control diffusion models by adding extra conditions",
        "shape": "anything",
    },
}

# Pass singleton objects to our subcommands
train.client = client
eval.client = client
train.db = db
eval.db = db


@app.callback()
def main():
    """
    The Rubbrband CLI allows you to train and evaluate models.
    """
    pass


@app.command()
def models():
    """List all supported models"""
    typer.echo("Supported Models:")
    typer.echo(f"{'NAME':12} DESCRIPTION")
    for key, val in db.items():
        typer.echo(f"{key:12} {val['description']}")


@app.command()
def ls():
    """List all running models"""
    typer.echo("Running Models:")
    containers = client.containers.list()

    # filter containers that start with rb-
    for container in containers:
        if container.name.startswith("rb-"):
            typer.echo(container.name)


@app.command()
def launch(model: str):
    """
    Launch a new MODEL.

    MODEL is the name of the model to launch.

    Example: rubbrband launch lora
    """

    if model not in db:
        typer.echo("Model not found")
        return

    with yaspin() as sp:
        sp.text = "Setting Up Environment. This may take up to 10 minutes."
        image_name = f"rubbrband/{model}"
        # pull image if not already pulled
        try:
            client.images.get(image_name)
        except docker.errors.ImageNotFound:
            client.images.pull(image_name)

    with yaspin() as sp:
        sp.text = f"Finished. Run rubbrband train {model} to train this model on sample data."


@app.command()
def enter(model: str):
    """
    Enter into a running MODEL.

    MODEL is the name of the model.

    Example: rubbrband enter lora
    """
    # if container not running, start it
    with yaspin() as sp:
        sp.text = "Launching Docker Container"

        container_name = f"rb-{model}"
        try:
            container = client.containers.get(container_name)
        except docker.errors.NotFound:
            client.containers.run(container_name, detach=True, name=container_name, tty=True, stdin_open=True)

        container = client.containers.get(container_name)

        if container.status != "running":
            container.start()

    subprocess.run(["docker", "exec", "-it", container_name, "/bin/bash"])


if __name__ == "__main__":
    app()



import os
import subprocess

import docker
import typer
from yaspin import yaspin

from rubbrband.clients.docker_client import image_pull

db = {}
client = None
app = typer.Typer(no_args_is_help=True)


@app.callback()
def main():
    """
    Train a MODEL :robot:

    Example: rubbrband train lora
    """


@app.command(rich_help_panel="Models :robot:", help="Stable diffusion models, trained with dreambooth method")
def dreambooth(
    ctx: typer.Context,
    class_name: str = typer.Option(
        ..., help="The name that you want to give to the class of images that you'll want to generate"
    ),
    regulation_prompt: str = typer.Option(
        ..., help="The prompt to regulate the images. Try to describe the type of images you want to generate"
    ),
    dataset_dir: str = typer.Option(..., help="The full path that contains the images you want to finetune on"),
    logdir: str = "experiment_logs",
):
    train(ctx, "dreambooth")


@app.command(rich_help_panel="Models :robot:", help="Low-rank adapation for efficient stable diffusion fine tuning")
def lora(
    ctx: typer.Context,
    dataset_dir: str = typer.Option(..., help="The full path that contains the images you want to finetune on"),
):
    train(ctx, "lora")


@app.command(rich_help_panel="Models :robot:", help="Control diffusion models by adding extra conditions")
def control(
    ctx: typer.Context,
    dataset_dir: str = typer.Option(..., help="The full path that contains the images you want to finetune on"),
):
    image_name = "rubbrband/control"
    container_name = "rb-control"

    try:
        client.images.get(image_name)
    except docker.errors.ImageNotFound:
        typer.echo("Model not found locally, pulling model.")
        image_pull(image_name)

    abs_path = os.path.abspath(ctx.params["dataset_dir"])

    try:
        container = client.containers.get(container_name)

        # stop and remove container if it is already running
        if container.status == "running":
            container.stop()
            container.remove()

    except docker.errors.NotFound:
        pass
    this_dir = os.path.dirname(os.path.abspath(__file__))

    subprocess.run(["chmod", "a+x", f"{this_dir}/models/control/train.sh"])
    # ctx.args is a list of arguments passed to the train command
    subprocess.run(["/bin/bash", f"{this_dir}/models/control/train.sh", abs_path])


# '''name''' corresponds to the name column in db.csv
# the option '''-d''' or '''--dataset_dir''' is the path to the dataset directory
# this directory gets mounted to the container at /home/engineering/data
def train(ctx: typer.Context, model: str):
    """Entrypoint for training a model."""
    if model not in db:
        typer.echo("Model not found")
        return

    image_name = f"rubbrband/{model}"
    container_name = f"rb-{model}"

    try:
        client.images.get(image_name)
    except docker.errors.ImageNotFound:
        typer.echo("Model not found locally, pulling model.")
        image_pull(image_name)

    abs_path = os.path.abspath(ctx.params["dataset_dir"])

    try:
        container = client.containers.get(container_name)

        # stop and remove container if it is already running
        if container.status == "running":
            container.stop()
            container.remove()

    except docker.errors.NotFound:
        pass
    try:
        subprocess.check_output("nvidia-smi")
        device_requests = [docker.types.DeviceRequest(count=-1, capabilities=[["gpu"]])]
    except (
        Exception
    ):  # this command not being found can raise quite a few different errors depending on the configuration
        device_requests = []

    client.containers.run(
        image_name,
        device_requests=device_requests,
        detach=True,
        name=container_name,
        volumes={f"{abs_path}": {"bind": "/home/engineering/data", "mode": "rw"}},
        tty=True,
        stdin_open=True,
    )
    container = client.containers.get(f"rb-{model}")

    if container.status != "running":
        container.start()

    this_dir = os.path.dirname(os.path.abspath(__file__))

    # Convert the parameters to a list of strings
    params = []
    for key, value in ctx.params.items():
        params.append(f"--{key}")
        params.append(value)

    with yaspin():
        subprocess.run(["chmod", "a+x", f"{this_dir}/models/{model}/train.sh"])
        # ctx.args is a list of arguments passed to the train command
        subprocess.run(["/bin/bash", f"{this_dir}/models/{model}/train.sh"] + params)


if __name__ == "__main__":
    app()



import docker
from rich.progress import Progress

tasks = {}
DOCKER_CLIENT = docker.from_env()


# Show task progress (red for download, green for extract)
def show_progress(line, progress):
    if line["status"] == "Downloading":
        id = f'[red][Download {line["id"]}]'
    elif line["status"] == "Extracting":
        id = f'[green][Extract  {line["id"]}]'
    else:
        # skip other statuses
        return

    if id not in tasks.keys():
        tasks[id] = progress.add_task(f"{id}", total=line["progressDetail"]["total"])
    else:
        progress.update(tasks[id], completed=line["progressDetail"]["current"])


def image_pull(image_name):
    with Progress() as progress:
        resp = DOCKER_CLIENT.api.pull(image_name, stream=True, decode=True)
        for line in resp:
            show_progress(line, progress)



import os
import subprocess

import docker
import typer
from yaspin import yaspin

db = {}
client = None
app = typer.Typer(no_args_is_help=True)


@app.callback()
def main():
    """
    Evaluate a trained MODEL :robot:

    Example: rubbrband eval lora
    """
    pass


@app.command(rich_help_panel="Models :robot:", help="Stable diffusion models, trained with dreambooth method")
def dreambooth(
    ctx: typer.Context,
    input_prompt: str = typer.Option(..., help="The prompt to input into the trained model."),
):
    eval(ctx, "dreambooth")


@app.command(rich_help_panel="Models :robot:", help="Low-rank adapation for efficient stable diffusion fine tuning")
def lora(
    ctx: typer.Context,
    input_prompt: str = typer.Option(..., help="The prompt to input into the trained model."),
):
    eval(ctx, "lora")


@app.command(rich_help_panel="Models :robot:", help="Low-rank adapation for efficient stable diffusion fine tuning")
def control(
    ctx: typer.Context,
    annotator_type: str = typer.Option(..., help="The edge detector to use."),
):
    valid_annotators = ["canny", "depth", "hed", "mlsd", "normal", "openpose", "scribble", "seg"]

    if annotator_type not in valid_annotators:
        typer.echo(f"Invalid annotator type. Valid types are: {', '.join(valid_annotators)}")
        return

    this_dir = os.path.dirname(os.path.abspath(__file__))

    with yaspin():
        typer.echo(["/bin/bash", f"{this_dir}/models/control/infer.sh", annotator_type])
        subprocess.run(["chmod", "a+x", f"{this_dir}/models/control/infer.sh"])
        subprocess.run(["/bin/bash", f"{this_dir}/models/control/infer.sh", annotator_type])


def eval(ctx: typer.Context, model: str):
    """Entrypoint for evaluating a model."""

    try:
        container = client.containers.get(f"rb-{model}")
    except docker.errors.NotFound:
        typer.echo(f"Container rb-{model} does not exist. Please train the model first.")
        return

    if container.status != "running":
        container.start()

    this_dir = os.path.dirname(os.path.abspath(__file__))

    # Convert the parameters to a list of strings
    params = []
    for key, value in ctx.params.items():
        params.append(f"--{key}")
        params.append(value)

    with yaspin():
        typer.echo(["/bin/bash", f"{this_dir}/models/{model}/infer.sh"] + params)
        subprocess.run(["chmod", "a+x", f"{this_dir}/models/{model}/infer.sh"])
        # add a parameter for prompt
        subprocess.run(["/bin/bash", f"{this_dir}/models/{model}/infer.sh"] + params)

        # move file from docker container to local directory
        # the file is located at /home/engineering/samples/output.jpg
        # the file is moved to the current directory
        subprocess.run(["docker", "cp", f"rb-{model}:/home/engineering/samples/output.jpg", "."])

    typer.echo("Inference complete. Check the current directory for the output image.")


if __name__ == "__main__":
    app()



from setuptools import find_packages, setup

setup(
    name="rubbrband",
    install_requires=["docker==6.0.1", "yaspin", "typer==0.7.0"],
    include_package_data=True,
    packages=find_packages(),
)
